import xarray as xr
import numpy as np
from pathlib import Path
import numpy as np
import time
import datetime
import sys
import os
from C3S512.cds import downloader

class downloading:        
    def __init__(self,
                 variable,
                 output_base_path = '/data/cds_archives2/',
                 cds_forms_location = '/data/'
                 ):
        self.variable = variable
        self.output_base_path = Path(output_base_path)
        self.cds_forms_location = Path(cds_forms_location)

    def era5_downloader(self,
                        dataset = 'reanalysis-era5-single-levels-monthly-means',
                        product_type = 'monthly_averaged_ensemble_members',
                        years = [1979,2019],
                        months = [1,13],
                        frequency = 'monthly',
                        levels = None,
                        file_format = 'netcdf'
                       ):
        if 'pressure' in dataset:
            if levels is None:
                print('WARNING: Not pressure levels selected!!!')
                levels_name = 'pressure_level'
                
        output_path = self.output_base_path
        output_path = output_path.joinpath(dataset).joinpath(product_type).joinpath(self.variable)

        if 'monthly' in dataset:
            time_param = '00:00'
            day_param = '01'
        else:
            time_param = ['{:02}:00'.format(i) for i in range(0,24)]
            day_param = ['{:02}'.format(i) for i in range(1,32)]

        if frequency is 'yearly': #Retrieve data in yearly files
            output_path = output_path.joinpath('yearly')
            years_param = [str(yy) for yy in range(years[0], years[1])]
            months_param = [['{:02d}'.format(mm) for mm in range(months[0], months[1])]]

        if frequency is 'monthly':
            output_path = output_path.joinpath('monthly')
            years_param = [str(yy) for yy in range(years[0], years[1])]
            months_param = ['{:02d}'.format(mm) for mm in range(months[0], months[1])]

        if frequency is None:
            years_param = [[str(yy) for yy in range(years[0], years[1])]] 
            months_param = [['{:02d}'.format(mm) for mm in range(months[0], months[1])]]

        if not os.path.exists(output_path):
            os.makedirs(output_path)
        
        for yy in years_param:
            for mm in months_param:
                if frequency is 'yearly':
                    mm_name = ''
                    yy_name = yy
                if frequency is 'monthly':
                    mm_name = mm
                    yy_name = yy
                if frequency is None:
                    mm_name = ''
                    yy_name = '{}to{}'.format(years[0], years[1]-1)
                
                download_file = '{}_{}{}.nc'.format(self.variable, yy_name, mm_name)
                download_path = output_path.joinpath(download_file)
                download_form ={
                    'format': file_format,
                    'product_type': product_type,
                    'variable': self.variable,
                    'year': yy,
                    'month': mm,
                    'day': day_param,
                    'time': time_param
                    }
                
                if 'pressure' in dataset:
                    download_form[levels_name] = levels
                    
                print('Requesting...{}.'.format(download_form))
                
                # Call retrieve and download the file
                if ~os.path.exists(download_path):

                    downloader(dataset,
                               download_form,
                               download_path
                              )
                    print('{} downloaded.'.format(download_path))
                
                else:
                    print('{} download skipped.'.format(download_path))
                
        
